import json
import logging

__all__ = ["bind_pyramid", "bind_flask"]

logger = logging.getLogger(__name__)


def bind_pyramid(config, log_maxsize=5000):
    import pyramid.events

    def log(logline):
        if len(logline) > log_maxsize:
            logline = logline[:log_maxsize] + "\n(...) [full body not shown]"
        logger.info(logline)

    def keep_a_copy_of_400_error_messages(event):
        if event.response.status_code >= 400:
            try:
                log("Request\n{}".format(json.dumps(event.request.json_body, indent=4)))
            except Exception:
                pass

        if 400 <= event.response.status_code < 500:
            try:
                log(
                    "Response\n{}".format(
                        json.dumps(event.response.json_body, indent=4)
                    )
                )
            except Exception:
                pass

    config.add_subscriber(keep_a_copy_of_400_error_messages, pyramid.events.NewResponse)


def bind_flask(app, log_maxsize):
    import flask

    def log(logline):
        if len(logline) > log_maxsize:
            logline = logline[:log_maxsize] + "\n(...) [full body not shown]"
        logger.info(logline)

    def keep_a_copy_of_400_error_messages(response):
        if response.status_code >= 400:
            try:
                log("Request\n{}".format(json.dumps(flask.request.json, indent=4)))
            except Exception:  # pragma: no cover
                pass

        if 400 <= response.status_code < 500:
            try:
                log(
                    "Response\n{}".format(
                        json.dumps(json.loads(response.data), indent=4)
                    )
                )
            except Exception:
                pass

        return response

    app.after_request(keep_a_copy_of_400_error_messages)



import json
import logging

__all__ = ["bind_flask", "bind_pyramid"]

logger = logging.getLogger(__name__)


def bind_flask(app, get_user_id):
    import flask

    def logger():
        try:
            # TODO provide a proper API for this kind of stuff
            if flask.request.path.startswith("/static"):
                return

            user_id = get_user_id()
            if user_id:
                log_request_info(
                    flask.request.path, user_id, flask.request.args.to_dict(flat=False)
                )
        except Exception:
            pass

    app.before_request(logger)


def bind_pyramid(config, get_user_id):
    import pyramid.events

    def logger(event):
        try:
            # TODO provide a proper API for this kind of stuff
            if event.request.path.startswith("/static"):
                return

            user_id = get_user_id(event)
            if user_id:
                log_request_info(
                    event.request.path, user_id, event.request.GET.dict_of_lists()
                )
        except Exception:
            pass

    config.add_subscriber(logger, pyramid.events.NewRequest)


def log_request_info(url, user_id, args):
    """Adds a log entry that fully represents the current request."""

    def flatten(dict_of_lists):
        return {
            key: value[0] if len(value) == 1 else value
            for key, value in dict_of_lists.items()
        }

    logging.info(
        "[FullRequestInfo] %s user=%s args=%s", url, user_id, json.dumps(flatten(args))
    )



"""
This timeout module is useful on deployment environments that implement a timeout on
requests. Usually this timeout is implemented on the HTTP layer, thus we have no
information on runtime (for logging, metrics, etc.) that the request timed out.

What this module does is raise a `DeadlineExceededError` exception so this error can
show up in error aggregators.
"""

import functools
import time

__all__ = ["DeadlineExceededError", "bind_pyramid", "bind_flask"]


class DeadlineExceededError(Exception):
    pass


class _FakeTimer:
    def start(self):
        pass

    def close(self):
        pass


def _timeout(seconds):
    try:
        import gevent

        return gevent.Timeout(seconds, DeadlineExceededError)
    except ImportError:
        return _FakeTimer()


def bind_flask(app):
    timeout_seconds = int(app.config.get("gevent_timeout", "30"))

    def wrap_dispatch_request(f):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):  # pragma: no cover
            timeout = _timeout(timeout_seconds)
            timeout.start()
            start_time = time.time()
            try:
                result = f(*args, **kwargs)
                if time.time() - start_time > timeout_seconds:
                    raise DeadlineExceededError()
                return result
            finally:
                timeout.close()

        return wrapper

    app.dispatch_request = wrap_dispatch_request(app.dispatch_request)


def bind_pyramid(config):
    config.add_tween("glibs.www.timeout.pyramid_tween")


def pyramid_tween(handler, registry):
    timeout_seconds = int(registry.settings.get("gevent_timeout", "30"))

    def request_timeout(request):
        timeout = _timeout(timeout_seconds)
        timeout.start()
        start_time = time.time()
        try:
            result = handler(request)
            if time.time() - start_time > timeout_seconds:
                raise DeadlineExceededError()
            return result
        finally:
            timeout.close()

    return request_timeout
